/******************************************************************************
 *  NEOPOP - Emulator as in Dreamland
 *  Copyright (C) 2001-2002 neopop_uk
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * File Selection
 ***************************************************************************/ 
//#include <sdcard.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/dir.h>
#include <sys/stat.h>
#include <unistd.h>
#include "neopop.h"
#include "dvd.h"
#include "iso9660.h"
#include "font.h"
#include "unzip.h"
#include <wiiuse/wpad.h>

#define PAGESIZE 12
#define PADCAL 70

static int maxfiles;
u8 havedir = 0;
u8 haveSDdir = 0;
u8 UseSDCARD = 0;
FILE * filehandle;
char rootSDdir[MAXPATHLEN];
int LoadFile (unsigned char *buffer);
int offset = 0;
int selection = 0;
int old_selection = 0;
int old_offset = 0;

extern int neosize;
extern unsigned char *neorom;

/***************************************************************************
 * Showfile screen
 ***************************************************************************/
static void ShowFiles (int offset, int selection) 
{
  int i, j;
  char text[MAXJOLIET+2];

  ClearScreen ();
  j = 0;

  for (i = offset; i < (offset + PAGESIZE) && (i < maxfiles); i++)
  {
	  memset(text,0,MAXJOLIET+2);
	  if (filelist[i].flags) sprintf(text, "[%s]", filelist[i].filename + filelist[i].filename_offset);
	  else sprintf (text, "%s", filelist[i].filename + filelist[i].filename_offset);

      if (j == (selection - offset)) WriteCentre_HL ((j * fheight) + 120, text);
      else WriteCentre ((j * fheight) + 120, text);
	  j++;
  }
  SetScreen ();
}


/***************************************************************************
* FileSortCallback
*
* Quick sort callback to sort file entries with the following order:
*  .
*  ..
*  <dirs>
*  <files>
***************************************************************************/
static int FileSortCallback(const void *f1, const void *f2)
{
	/* Special case for implicit directories */
	if(((FILEENTRIES *)f1)->filename[0] == '.' || ((FILEENTRIES *)f2)->filename[0] == '.')
	{
		if(strcmp(((FILEENTRIES *)f1)->filename, ".") == 0) { return -1; }
		if(strcmp(((FILEENTRIES *)f2)->filename, ".") == 0) { return 1; }
		if(strcmp(((FILEENTRIES *)f1)->filename, "..") == 0) { return -1; }
		if(strcmp(((FILEENTRIES *)f2)->filename, "..") == 0) { return 1; }
	}
   
	/* If one is a file and one is a directory the directory is first. */
	if(((FILEENTRIES *)f1)->flags == 1 && ((FILEENTRIES *)f2)->flags == 0) return -1;
	if(((FILEENTRIES *)f1)->flags == 0 && ((FILEENTRIES *)f2)->flags == 1) return 1;
   
	return stricmp(((FILEENTRIES *)f1)->filename, ((FILEENTRIES *)f2)->filename);
}

/***************************************************************************
 * Update SDCARD curent directory name 
 ***************************************************************************/ 
int updateSDdirname()
{
  int size=0;
  char *test;
  char temp[1024];

   /* current directory doesn't change */
   if (strcmp(filelist[selection].filename,".") == 0) return 0; 
   
   /* go up to parent directory */
   else if (strcmp(filelist[selection].filename,"..") == 0) 
   {
     /* determine last subdirectory namelength */
     sprintf(temp,"%s",rootSDdir);
     test= strtok(temp,"/");
     while (test != NULL){ 
       size = strlen(test);
       test = strtok(NULL,"/");
     }
  
     /* remove last subdirectory name */
     size = strlen(rootSDdir) - size - 1;
     rootSDdir[size] = 0;

	 /* handles root name */
        if (strcmp(rootSDdir, "/") == 0)
            sprintf(rootSDdir,"fat:/");
	 
     return 1;
   }
   else
   {
     /* test new directory namelength */
     if ((strlen(rootSDdir)+1+strlen(filelist[selection].filename)) < MAXPATHLEN) 
     {
       /* handles root name */
//	   if (strcmp(rootSDdir,"dev0:\\neopop\\..") == 0) sprintf(rootSDdir,"dev0:");
       sprintf(temp, "/neopop/..");
       if (strcmp(rootSDdir, temp) == 0) 
            sprintf(rootSDdir,"fat:/");
	 
	   /* update current directory name */
       //sprintf(rootSDdir, "%s\\%s",rootSDdir, filelist[selection].filename);
       sprintf(rootSDdir, "%s/%s",rootSDdir, filelist[selection].filename);
       return 1;
     }
     else
     {
         WaitPrompt ("Dirname is too long !"); 
         return -1;
     }
   } 
}

/***************************************************************************
 * Browse SDCARD subdirectories 
 ***************************************************************************/ 
int parseSDdirectory()
{
  int numstored = 0;
    DIR_ITER *sddir;
    char filename[MAXPATHLEN];
    struct stat filestat;
	char msg[128];
    
    /* initialize selection */
    selection = offset = 0;

    /* open the directory */ 
    sddir = diropen(rootSDdir);
	if (sddir == NULL) {
        strcpy(rootSDdir, "fat:/");
        sddir = diropen(rootSDdir);
        if (sddir == NULL) {
            sprintf(msg, "Error opening %s", rootSDdir);
            WaitPrompt(msg);
            return 0;
        }
    }

    /* Move to DVD structure - this is required for the file selector */ 
    while(dirnext(sddir,filename,&filestat) == 0) {
        if(strcmp(filename,".") != 0) {
            memset(&filelist[numstored], 0, sizeof(FILEENTRIES));
            strncpy(filelist[numstored].filename, filename, MAXPATHLEN);
            filelist[numstored].length = filestat.st_size;
            filelist[numstored].flags = (filestat.st_mode & _IFDIR) == 0 ? 0 : 1;
            numstored++;
        }
    }

    /*** close directory ***/
    dirclose(sddir);
	
	/* Sort the file list */
 	qsort(filelist, numstored, sizeof(FILEENTRIES), FileSortCallback);

    return numstored;
}

/****************************************************************************
 * FileSelector
 *
 * Let user select a file from the File listing
 ****************************************************************************/
void FileSelector () 
{
	//short p;
	u32 p;
    signed char a,b;
	int haverom = 0;
	int redraw = 1;
	int go_up = 0;
	int i,size;
  
	while (haverom == 0)
	{
		if (redraw) ShowFiles (offset, selection);
		redraw = 0;
		p = WPAD_ButtonsDown (0);
		a = PAD_StickY (0);
		b = PAD_StickX (0);

		/*
		 * check selection screen changes
		 */		

		/* scroll displayed filename */
  		if ((p & WPAD_BUTTON_UP) || (b < -PADCAL) || (p & WPAD_CLASSIC_BUTTON_LEFT))
		{
			if (filelist[selection].filename_offset > 0)
			{
				filelist[selection].filename_offset --;
				redraw = 1;
			}
		}
		else if ((p & WPAD_BUTTON_DOWN) || (b > PADCAL) || (p & WPAD_CLASSIC_BUTTON_RIGHT))
		{
			size = 0;
			for (i=filelist[selection].filename_offset; i<strlen(filelist[selection].filename); i++)
				size += font_size[(int)filelist[selection].filename[i]];
		  
			if (size > back_framewidth)
			{
				filelist[selection].filename_offset ++;
				redraw = 1;
			}
		}

		/* highlight next item */
		else if ((p & WPAD_BUTTON_LEFT) || (a < -PADCAL) || (p & WPAD_CLASSIC_BUTTON_DOWN))
		{
			filelist[selection].filename_offset = 0;
			selection++;
			if (selection == maxfiles) selection = offset = 0;
			if ((selection - offset) >= PAGESIZE) offset += PAGESIZE;
			redraw = 1;
		}

		/* highlight previous item */
		else if ((p & WPAD_BUTTON_RIGHT) || (a > PADCAL) || (p & WPAD_CLASSIC_BUTTON_UP))
		{
			filelist[selection].filename_offset = 0;
			selection--;
			if (selection < 0)
			{
				selection = maxfiles - 1;
				offset = selection - PAGESIZE + 1;
			}
			if (selection < offset) offset -= PAGESIZE;
			if (offset < 0) offset = 0;
			redraw = 1;
		}
      
		/* go back one page */
		else if ((p & WPAD_BUTTON_MINUS) || (p & WPAD_CLASSIC_BUTTON_FULL_R))
		{
			filelist[selection].filename_offset = 0;
			selection -= PAGESIZE;
			if (selection < 0)
			{
				selection = maxfiles - 1;
				offset = selection - PAGESIZE + 1;
			}
			if (selection < offset) offset -= PAGESIZE;
			if (offset < 0) offset = 0;
			redraw = 1;
		}

		/* go forward one page */
		else if ((p & WPAD_BUTTON_PLUS) || (p & WPAD_CLASSIC_BUTTON_FULL_L))
		{
			filelist[selection].filename_offset = 0;
			selection += PAGESIZE;
			if (selection > maxfiles - 1) selection = offset = 0;
			if ((selection - offset) >= PAGESIZE) offset += PAGESIZE;
			redraw = 1;
		}
		
		/*
		 * Check pressed key
		 */
		 
		/* go up one directory or quit */
		if ((p & WPAD_BUTTON_2) || (p & WPAD_CLASSIC_BUTTON_B))
		{
			filelist[selection].filename_offset = 0;
			if (((!UseSDCARD) && (basedir == rootdir)) ||
				(UseSDCARD && strcmp(rootSDdir,"/neopop/..") == 0)) return;
			go_up = 1;
		}

		/* quit */
		if ((p & WPAD_BUTTON_HOME) || (p & WPAD_CLASSIC_BUTTON_HOME))
		{
			filelist[selection].filename_offset = 0;
			return;
		}

		/* open selected file or directory */
		if ((p & WPAD_BUTTON_1) || go_up || (p & WPAD_CLASSIC_BUTTON_A))
		{
			filelist[selection].filename_offset = 0;
			if (go_up)
			{
				go_up = 0;
				selection = 1;
			}

			/*** This is directory ***/
			if (filelist[selection].flags)
			{
				if (UseSDCARD) /* SDCARD directory handler */
				{
					/* update current directory */
					int status = updateSDdirname();

					/* move to new directory */
					if (status == 1)
					{
						/* reinit selector (previous value is saved for one level) */
						if (selection == 1)
						{
							selection = old_selection;
							offset = old_offset;
							old_selection = 0;
							old_offset = 0;
						}
						else
						{
							/* save current selector value */
							old_selection = selection;
							old_offset = offset;
							selection = 0;
							offset = 0;
						}
						

						/* set new entry list */
						maxfiles = parseSDdirectory();
						if (!maxfiles)
						{
							/* quit */
							WaitPrompt ("Error reading directory !");
							haverom   = 1;
							haveSDdir = 0;
						}
					}
					else if (status == -1)
					{
						/* quit */
						haverom   = 1;
						haveSDdir = 0;
					}
				}
#ifdef __gamecube__
				else /* DVD directory handler */
				{
					/* move to a new directory */
					if (selection != 0)
					{
						/* update current directory */
						rootdir = filelist[selection].offset;
						rootdirlength = filelist[selection].length;
				  
						/* reinit selector (previous value is saved for one level) */
						if (selection == 1)
						{
							selection = old_selection;
							offset = old_offset;
							old_selection = 0;
							old_offset = 0;
						}
						else
						{
							/* save current selector value */
							old_selection = selection;
							old_offset = offset;
							selection = 0;
							offset = 0;
						}

						/* get new entry list */
						maxfiles = parsedirectory ();
					}
				}
#endif
			}
			else /*** This is a file ***/
	    	{	
	      		rootdir = filelist[selection].offset;
	      		rootdirlength = filelist[selection].length;
          		neosize = LoadFile (&neorom[0]);
	      		rom.length = neosize;
	      		rom.data = (char *) &neorom[0];
	      		rom_loaded ();
	      		reset ();
	      		haverom = 1;
	    	}
	  		redraw = 1;
		}
    }
}

#ifdef __gamecube__
/****************************************************************************
 * OpenDVD
 *
 * Function to load a DVD directory and display to user.
 ****************************************************************************/
void OpenDVD () 
{
  UseSDCARD = 0;
  if (!getpvd())
  {
	ShowAction("Mounting DVD ... Wait");
	DVD_Mount();
	havedir = 0;
	if (!getpvd())
	{
		WaitPrompt ("Failed to mount DVD");
        return;
	}
  }
  
  if (havedir == 0)
  {
     /* don't mess with SD entries */
	 haveSDdir = 0;
	 
	 /* reinit selector */
     rootdir = basedir;
	 old_selection = selection = offset = old_offset = 0;
	 
     if ((maxfiles = parsedirectory ()))
     {
	   FileSelector ();
	   havedir = 1;
	 }
  }
  else FileSelector ();
}
#endif

/****************************************************************************
 * OpenSD updated to use the new libogc.  Written by softdev and pasted
 * into this code by Drack.
 * Modified for subdirectory browing & quick filelist recovery
 * Enjoy!
*****************************************************************************/
int OpenSD () {
  UseSDCARD = 1;
  char msg[128];
  
  if (haveSDdir == 0)
  {
#ifdef __gamecube__
     /* don't mess with DVD entries */
	 havedir = 0;
#endif
     /* reinit selector */
	 old_selection = selection = offset = old_offset = 0;
	
     /* Reset SDCARD root directory */
     sprintf(rootSDdir,"/neopop/roms");
 
	 /* Parse initial root directory and get entries list */
     ShowAction ((char*)"Reading Directory ...");
	 if ((maxfiles = parseSDdirectory ()))
	 {
       /* Select an entry */
	   FileSelector ();
    
       /* memorize last entries list, actual root directory and selection for next access */
	   haveSDdir = 1;
	 }
	 else
	 {
		/* no entries found */
        sprintf (msg, "Error reading %s", rootSDdir);
		ShowAction (msg);
		return 0;
	 }
  }
  /* Retrieve previous entries list and made a new selection */
  else  FileSelector ();
  
  return 1;
}

/****************************************************************************
 * SDCard Get Info
 ****************************************************************************/ 
void GetSDInfo () 
{
    char fname[MAXPATHLEN];
    rootdirlength = 0;
 
    /* Check filename length */
    if ((strlen(rootSDdir)+1+strlen(filelist[selection].filename)) < MAXPATHLEN)
        sprintf(fname, "%s/%s",rootSDdir,filelist[selection].filename); 
  
    else
    {
        WaitPrompt ((char*)"Maximum Filename Length reached !"); 
        haveSDdir = 0; // reset everything before next access
    }

    filehandle = fopen(fname, "rb");
    if (filehandle == NULL)
    {
      WaitPrompt ((char*)"Unable to open file!");
      return;
    }
    fseek(filehandle, 0, SEEK_END);
    rootdirlength = ftell(filehandle);
    fseek(filehandle, 0, SEEK_SET);
}

/****************************************************************************
 * LoadFile
 *
 * This function will load a file from DVD or SDCARD, in BIN, SMD or ZIP format.
 * The values for offset and length are inherited from rootdir and 
 * rootdirlength.
 *
 * The buffer parameter should re-use the initial ROM buffer.
 ****************************************************************************/
int LoadFile (unsigned char *buffer) 
{
  int offset;
  int blocks;
  int i;
  u64 discoffset;
  char readbuffer[2048];

  /* SDCard Addition */ 
  if (UseSDCARD) GetSDInfo ();
  if (rootdirlength == 0) return 0;
    
  /* How many 2k blocks to read */ 
  blocks = rootdirlength / 2048;
  offset = 0;
  discoffset = rootdir;
  
  ShowAction ((char*)"Loading ... Wait");
  
  if (UseSDCARD) fread(readbuffer, 1, 2048, filehandle);
  else dvd_read (&readbuffer, 2048, discoffset);

  if (!IsZipFile ((char *) readbuffer))
  {
    if (UseSDCARD) fseek(filehandle, 0, SEEK_SET);
    
    for (i = 0; i < blocks; i++){
	  if (UseSDCARD) fread(readbuffer, 1, 2048, filehandle);
	  else dvd_read (readbuffer, 2048, discoffset);
	  memcpy (buffer + offset, readbuffer, 2048);
	  offset += 2048;
	  discoffset += 2048;
	}

    /* And final cleanup */ 
	if (rootdirlength % 2048)
	{
	  i = rootdirlength % 2048;
	  if (UseSDCARD) fread(readbuffer, 1, i, filehandle);
	  else dvd_read (readbuffer, 2048, discoffset);
	  memcpy (buffer + offset, readbuffer, i);
	}
  }
  else return UnZipBuffer (buffer, discoffset, rootdirlength);
  
  if (UseSDCARD) fclose(filehandle);

  return rootdirlength;
}
